/*
Copyright (C) Christopher Fogelberg, Mengjie Zhang 2005

This file is part of VUWLGP.

VUWLGP is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

VUWLGP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VUWLGP; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
  
#include <iostream>

#include "IInstructionArgument.h"
#include "InstructionArgumentFeature.h"
#include "InstructionArgumentRegister.h"
#include "InstructionArgumentConstant.h"

int main(int argc, char** argv) {
  std::cout << "Testing to-string functionality and random generation 5 times:" << std::endl;

  VUWLGP::IInstructionArgument<double>* cOrig;
  VUWLGP::IInstructionArgument<double>* fOrig;
  VUWLGP::IInstructionArgument<double>* rOrig;
  VUWLGP::IInstructionArgument<double>* cCopy;
  VUWLGP::IInstructionArgument<double>* fCopy;
  VUWLGP::IInstructionArgument<double>* rCopy;

  VUWLGP::Config<double>* config = new VUWLGP::Config<double>();
  config->numFeatures = 5;
  config->numRegisters = 10;

  VUWLGP::FitnessEnvironment<double> fe(config);
  fe.WriteRegister(0, 100);  
  fe.WriteRegister(1, 101);
  fe.WriteRegister(2, 102);
  fe.WriteRegister(3, 103);
  fe.WriteRegister(4, 104);
  fe.WriteRegister(5, 105);
  fe.WriteRegister(6, 106);
  fe.WriteRegister(7, 107);
  fe.WriteRegister(8, 108);
  fe.WriteRegister(9, 109);

  for(unsigned int i = 0; i < 5; ++i) {
    cOrig = VUWLGP::InstructionArgumentConstant<double>::Generate(config);
    fOrig = VUWLGP::InstructionArgumentFeature<double>::Generate(config);
    rOrig = VUWLGP::InstructionArgumentRegister<double>::Generate(config);

    std::cout << "\tConstant:\t" << cOrig->Value(fe) << "\t" << cOrig->ToString() << std::endl;
    std::cout << "\tFeature:\t" << "not-poss" << "\t" << fOrig->ToString() << std::endl;
    std::cout << "\tRegister:\t" << rOrig->Value(fe) << "\t" << rOrig->ToString() << std::endl;
    std::cout << "\n";
  }

  std::cout << "Testing cloning:";
  cOrig = VUWLGP::InstructionArgumentConstant<double>::Generate(config);
  fOrig = VUWLGP::InstructionArgumentFeature<double>::Generate(config);
  rOrig = VUWLGP::InstructionArgumentRegister<double>::Generate(config);
  
  cCopy = cOrig->Clone();
  fCopy = fOrig->Clone();
  rCopy = rOrig->Clone();

  std::cout << "\nConstant (orig=copy):\t" << cOrig->ToString() << " = " << cCopy->ToString();
  std::cout << "\nFeature (orig=copy):\t" << fOrig->ToString() << " = " << fCopy->ToString();
  std::cout << "\nRegister (orig=copy):\t" << rOrig->ToString() << " = " << rCopy->ToString();
  std::cout << std::endl;
  
  delete cOrig;
  delete fOrig;
  delete rOrig;
  delete cCopy;
  delete fCopy;
  delete rCopy;
  delete config;

  return 0;
}
