/*
Copyright (C) Christopher Fogelberg, Mengjie Zhang 2005

This file is part of VUWLGP.

VUWLGP is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

VUWLGP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VUWLGP; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
  
#include <iostream>
#include <string>
#include <map>

#include "WeightedCollection.h"

int main(int argc, char** argv) {
  std::cout << "Testing VUWLGP::WeightedCollection<T>" << std::endl;
  std::cout << "Selecting a random element from 4 1000 times and reporting the counts of each" 
	    << std::endl << std::endl;

  VUWLGP::Rand::Init();

  VUWLGP::WeightedCollection<std::string> w;
  try {
    w.AddElement("weight:-1", -1);
  }
  catch(std::string s) {
    std::cout << "Tried to add a negative weight and caught the error. Message:" << std::endl;
    std::cout << "\t" << s << std::endl << std::endl;
  }

  w.AddElement("weight:0", 0);
  w.AddElement("weight:1", 1);
  w.AddElement("weight:2", 2);
  w.AddElement("weight:3", 3);

  std::map<std::string, unsigned int> cnts;
  cnts["weight:0"] = 0;
  cnts["weight:1"] = 0;
  cnts["weight:2"] = 0;
  cnts["weight:3"] = 0;

  for(unsigned int i = 0; i < 1000; ++i) {
    ++cnts[w.GetRandomElement()];
  }

  for(std::map<std::string, unsigned int>::iterator i = cnts.begin(); i != cnts.end(); ++i) {
    std::cout << i->first << " = " << i->second << std::endl;
  }

  return 0;
}
